@@ -41,6 +41,8 @@ struct _EphyDownload {
   char *destination;
   char *content_type;
 
+  gboolean show_notification;
+
   EphyDownloadActionType action;
   guint32 start_time;
   gboolean finished;
@@ -602,6 +604,8 @@ ephy_download_init (EphyDownload *download)
   download->action = EPHY_DOWNLOAD_ACTION_NONE;
 
   download->start_time = gtk_get_current_event_time ();
+
+  download->show_notification = TRUE;
 }
 
 static void
@@ -682,6 +686,36 @@ download_created_destination_cb (WebKitDownload *wk_download,
   g_free (content_type);
 }
 
+static void
+display_download_finished_notification (WebKitDownload *download)
+{
+  GApplication *application;
+  GtkWindow *toplevel;
+  const char *dest;
+
+  application = G_APPLICATION (ephy_embed_shell_get_default ());
+  toplevel = gtk_application_get_active_window (GTK_APPLICATION (application));
+  dest = webkit_download_get_destination (download);
+
+  if (!gtk_window_is_active (toplevel) && dest != NULL) {
+    char *filename;
+    char *message;
+    GNotification *notification;
+
+    filename = g_filename_display_basename (dest);
+    /* Translators: a desktop notification when a download finishes. */
+    message = g_strdup_printf (_("Finished downloading %s"), filename);
+    /* Translators: the title of the notification. */
+    notification = g_notification_new (_("Download finished"));
+    g_notification_set_body (notification, message);
+    g_application_send_notification (application, "download-finished", notification);
+
+    g_free (filename);
+    g_free (message);
+    g_object_unref (notification);
+  }
+}
+
 static void
 download_finished_cb (WebKitDownload *wk_download,
                       EphyDownload   *download)
@@ -694,6 +728,9 @@ download_finished_cb (WebKitDownload *wk_download,
   else
     ephy_download_do_download_action (download, download->action, download->start_time);
 
+  if (download->show_notification)
+    display_download_finished_notification (wk_download);
+
   g_signal_emit (download, signals[COMPLETED], 0);
 }
 
@@ -772,3 +809,11 @@ ephy_download_new_for_uri (const char *uri)
 
   return ephy_download;
 }
+
+void
+ephy_download_disable_desktop_notification (EphyDownload *download)
+{
+  g_assert (EPHY_IS_DOWNLOAD (download));
+
+  download->show_notification = FALSE;
+}
